@@ -38,9 +38,6 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <gconf/gconf-client.h>
 
-#include <hildon-widgets/hildon-banner.h>
-#include <hildon-widgets/hildon-note.h>
-
 
 #define HILDON_HOME_IMAGE_FORMAT           "png"
 #define HILDON_HOME_IMAGE_ALPHA_FULL       255
