@@ -38,19 +38,30 @@
 
 G_DEFINE_TYPE (HildonDesktopPanelExpandable, hildon_desktop_panel_expandable, HILDON_DESKTOP_TYPE_PANEL);
 
+enum 
+{
+  SIGNAL_QUEUED_BUTTON,
+  N_SIGNALS
+};
+
+static gint signals[N_SIGNALS];
+
 struct _HildonDesktopPanelExpandablePrivate
 {
-  guint     items_p_row;
-  guint     n_items;
+  guint       items_p_row;
+  guint       n_items;
+  guint	      current_position;
 
-  GList     *items;
+  GHashTable *items;
 
-  GtkWindow *extension_window;
-  GtkTable  *extension_table;
+  GList	     *queued_items;
 
-  gboolean   extension_opened;
+  GtkWindow  *extension_window;
+  GtkTable   *extension_table;
 
-  GtkWidget *arrow;
+  gboolean    extension_opened;
+
+  GtkWidget  *arrow;
 };
 
 enum
@@ -81,6 +92,8 @@ GObject *hildon_desktop_panel_expandable_constructor (GType gtype,guint n_params
 
 static void hildon_desktop_panel_expandable_finalize (GObject *object);
 
+static void hildon_desktop_panel_expandable_arrange_items_cb (StatusbarItem *item, gboolean condition, gpointer _panel);
+
 static void hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *panel);
 
 static void hildon_desktop_panel_expandable_add_in_extension (HildonDesktopPanelExpandable *panel, HildonDesktopPanelItem *item);
@@ -121,6 +134,14 @@ hildon_desktop_panel_expandable_class_init (HildonDesktopPanelExpandableClass *p
                                                      7,
                                                      G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 
+  signals[SIGNAL_QUEUED_BUTTON] =
+        g_signal_new("queued-button",
+                     G_OBJECT_CLASS_TYPE(object_class),
+                     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+                     g_cclosure_marshal_VOID__OBJECT,
+                     G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
+
 }
 
 static void 
@@ -128,11 +149,21 @@ hildon_desktop_panel_expandable_init (HildonDesktopPanelExpandable *panel)
 {
   panel->priv = HILDON_DESKTOP_PANEL_EXPANDABLE_GET_PRIVATE (panel);
 
-  panel->priv->items_p_row = 0;
-  panel->priv->n_items = 0;
-  panel->priv->items = NULL;
+  panel->priv->items_p_row = 
+      panel->priv->n_items = 
+      panel->priv->current_position = 0;
+  
+  panel->priv->items = g_hash_table_new_full (g_str_hash,
+		  			      g_str_equal,
+					      (GDestroyNotify) g_free,
+					      (GDestroyNotify) gtk_widget_destroy);
 
+  panel->priv->queued_items = NULL;
+  
   panel->priv->extension_opened = FALSE;
+
+  panel->priv->extension_table = NULL;
+
 }
 
 GObject *
@@ -174,6 +205,8 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
 
   if (item_width == 0 || item_height == 0)
     g_object_set (object, "item_width", HSB_ITEM_WIDTH, "item_height", HSB_ITEM_HEIGHT, NULL);
+
+  HILDON_DESKTOP_PANEL (panel)->pack_start = FALSE;
   
   return object;
 }
@@ -185,6 +218,8 @@ hildon_desktop_panel_expandable_finalize (GObject *object)
 
   gtk_widget_destroy (GTK_WIDGET (panel->priv->extension_window));
 
+  g_hash_table_destroy (panel->priv->items);
+
   G_OBJECT_CLASS (hildon_desktop_panel_expandable_parent_class)->finalize (object);
 }
 
@@ -250,6 +285,13 @@ hildon_desktop_panel_expandable_set_property (GObject *object,
   }
 }
 
+static void 
+hildon_desktop_panel_expandable_button_destroyed (GtkWidget *widget, gpointer _panel)
+{
+  g_hash_table_remove (HILDON_DESKTOP_PANEL_EXPANDABLE (_panel)->priv->items,
+		       HILDON_DESKTOP_ITEM (widget)->id);
+}
+
 static void 
 hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget *button)
 {
@@ -257,19 +299,20 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
   HildonDesktopPanelExpandable *ex_panel = HILDON_DESKTOP_PANEL_EXPANDABLE (panel);
   gint item_width,item_height;
 
+  g_signal_emit_by_name (ex_panel, "queued-button", button);
+
   g_debug ("Adding button in expandable %d %d",ex_panel->priv->items_p_row,ex_panel->priv->n_items+1);
 
   g_object_get (G_OBJECT (panel), "item_width", &item_width, "item_height", &item_height, NULL);
   
   gtk_widget_set_size_request (button, item_width, item_height);
-  
-  if (STATUSBAR_IS_ITEM (button))
-  {
-    g_signal_connect (STATUSBAR_ITEM (item),
-		      "statusbar_condition",
-		      G_CALLBACK (hildon_desktop_panel_expandable_arrange_items),
-		      (gpointer)ex_panel);
-  }
+
+  g_signal_connect (button,
+		    "destroy",
+		    G_CALLBACK (hildon_desktop_panel_expandable_button_destroyed),
+		    (gpointer)panel);
+
+  g_object_set (G_OBJECT (item), "position", ex_panel->priv->current_position++, NULL);
 
   if ((ex_panel->priv->n_items+1) > ex_panel->priv->items_p_row)
   {
@@ -281,16 +324,42 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
       
       ex_panel->priv->arrow = arrow;
     }
-    
-    hildon_desktop_panel_expandable_add_in_extension (ex_panel, item);
+  
+    if (STATUSBAR_IS_ITEM (button) && !STATUSBAR_ITEM (button)->condition)
+      g_debug ("Statusbar: Item not visible %s",HILDON_DESKTOP_ITEM (button)->id);
+    else  
+      hildon_desktop_panel_expandable_add_in_extension (ex_panel, item);
   }
   else
   {
-    ex_panel->priv->n_items++;	  
-    HILDON_DESKTOP_PANEL_CLASS (hildon_desktop_panel_expandable_parent_class)->add_button (panel,button);
+   
+    if (STATUSBAR_IS_ITEM (button) && !STATUSBAR_ITEM (button)->condition)
+      g_debug ("Statusbar: Item not visible %s",HILDON_DESKTOP_ITEM (button)->id);
+    else
+    {
+      ex_panel->priv->n_items++;  
+      HILDON_DESKTOP_PANEL_CLASS (hildon_desktop_panel_expandable_parent_class)->add_button (panel,button);
+    }
+  }
+  
+  if (g_hash_table_lookup (ex_panel->priv->items, HILDON_DESKTOP_ITEM (item)->id) == NULL)
+  {
+    if (STATUSBAR_IS_ITEM (button))
+      g_signal_connect_after (STATUSBAR_ITEM (item),
+		              "hildon-status-bar-update-conditional",
+		              G_CALLBACK (hildon_desktop_panel_expandable_arrange_items_cb),
+		              (gpointer)ex_panel);
+	  
+    if (STATUSBAR_IS_ITEM (button) && !STATUSBAR_ITEM (button)->condition)
+    {	  
+      gtk_object_sink (GTK_OBJECT (item));
+      g_object_ref (G_OBJECT (item));
+    }
+    
+    g_hash_table_insert (ex_panel->priv->items,
+			 HILDON_DESKTOP_ITEM (item)->id,
+			 (gpointer)item);
   }
-
-  ex_panel->priv->items = g_list_append (ex_panel->priv->items, item);
 }
 
 static void 
@@ -349,14 +418,46 @@ hildon_desktop_panel_expandable_add_in_extension (HildonDesktopPanelExpandable *
   panel->priv->n_items++;
 }
 
+static void 
+hildon_desktop_panel_expandable_arrange_items_cb (StatusbarItem *item, gboolean condition, gpointer _panel)
+{
+  hildon_desktop_panel_expandable_arrange_items (HILDON_DESKTOP_PANEL_EXPANDABLE (_panel));
+}
+
+static void 
+hildon_desktop_panel_expandable_hash_adding (gchar *key, 
+					     HildonDesktopItem *item,
+					     HildonDesktopPanelExpandable *panel)
+{
+  panel->priv->queued_items = 
+    g_list_append (panel->priv->queued_items, item);
+}
+
+static gint 
+hildon_desktop_panel_expandable_sort_items (gconstpointer a, gconstpointer b)
+{
+   gint p1,p2;
+
+   g_object_get (G_OBJECT (a), "position", &p1, NULL);
+   g_object_get (G_OBJECT (b), "position", &p2, NULL);
+
+   if (p1 < p2) 
+     return -1;
+   else 
+   if (p1 == p2) 
+    return 0;
+   else 
+    return 1;
+}
+
 static void 
 hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *panel)
 {
   /*TODO: Improve this!! This horribly slow!!!!! */
-	
+  
   GList *children_panel, *children_table, *l;
 
-  children_panel = children_table = NULL;
+  panel->priv->queued_items = children_panel = children_table = NULL;
 
   children_panel = gtk_container_get_children (GTK_CONTAINER (panel));
   children_table = gtk_container_get_children (GTK_CONTAINER (panel->priv->extension_table));
@@ -377,21 +478,32 @@ hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *pan
 		    	  GTK_WIDGET (l->data));
   }
 
-  for (l = panel->priv->items; l ; l = g_list_next (l))
-  {
-    if (GTK_WIDGET_VISIBLE (l->data))
-    {
-      hildon_desktop_panel_expandable_add_button (HILDON_DESKTOP_PANEL (panel),
-		      				  GTK_WIDGET (l->data));  
-      g_object_unref (G_OBJECT (l->data));
-    }
-  }
+  panel->priv->n_items = panel->priv->current_position = 0;
+  
+  g_hash_table_foreach (panel->priv->items,
+		  	(GHFunc)hildon_desktop_panel_expandable_hash_adding,
+			(gpointer)panel);
+  
+  panel->priv->queued_items = 
+    g_list_sort (panel->priv->queued_items,
+     	         (GCompareFunc)hildon_desktop_panel_expandable_sort_items);
 
+  for (l = panel->priv->queued_items; l; l = g_list_next (l))
+  {
+    hildon_desktop_panel_expandable_add_button (HILDON_DESKTOP_PANEL (panel),
+		      			        GTK_WIDGET (l->data));
+
+    if (STATUSBAR_IS_ITEM (l->data) && !STATUSBAR_ITEM (l->data)->condition)
+      g_debug ("not unreffing");
+    else
+      g_object_unref (G_OBJECT (l->data));
+  }	  
+
+  g_list_free (panel->priv->queued_items);
   g_list_free (children_panel);
   g_list_free (children_table);
 }
 
-
 static GtkWidget * 
 hildon_desktop_panel_expandable_add_arrow (HildonDesktopPanelExpandable *panel)
 {
