@@ -51,6 +51,14 @@ enum
   PROP_MAG_HEIGHT
 };
 
+enum
+{
+  SIGNAL_ORIENTATION_CHANGED,
+  N_SIGNALS
+};
+
+static gint signals[N_SIGNALS];  
+
 typedef enum
 {
   HILDON_DESKTOP_PANEL_WINDOW_GRAB_NONE = 0,
@@ -93,22 +101,53 @@ struct _HildonDesktopPanelWindowPrivate
 
 /*static void set_focus_forall_cb (GtkWidget *widget, gpointer data);*/
 static void hildon_desktop_panel_window_finalize (GObject *self);
-static GObject *hildon_desktop_panel_window_constructor (GType gtype, guint n_params, GObjectConstructParam *params);
-static void hildon_desktop_panel_window_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);		   
-static void hildon_desktop_panel_window_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+
+static GObject *hildon_desktop_panel_window_constructor (GType gtype, 
+                                                         guint n_params, 
+                                                         GObjectConstructParam *params);
+
+static void hildon_desktop_panel_window_get_property (GObject *object, 
+                                                      guint prop_id, 
+                                                      GValue *value, 
+                                                      GParamSpec *pspec);		   
+
+static void hildon_desktop_panel_window_set_property (GObject *object, 
+                                                      guint prop_id, 
+                                                      const GValue *value, 
+                                                      GParamSpec *pspec);
+
 static void hildon_desktop_panel_window_check_resize (GtkContainer *container);
-static gboolean hildon_desktop_panel_press_button_event (GtkWidget *widget, GdkEventButton *event);
-static gboolean hildon_desktop_panel_release_button_event (GtkWidget *widget, GdkEventButton *event);
-static gboolean hildon_desktop_panel_motion_notify_event (GtkWidget *widget, GdkEventMotion *event);
-static void hildon_desktop_panel_size_request (GtkWidget *widget, GtkRequisition *requisition);
-static void hildon_desktop_panel_size_allocate (GtkWidget *widget, GtkAllocation  *allocation);
-static void hildon_desktop_panel_stop_grab (HildonDesktopPanelWindow *window, guint32 timestamp);
-static gboolean hildon_desktop_panel_grab_motion_event (HildonDesktopPanelWindow *window, GdkEventMotion *event);
-static void hildon_desktop_panel_calc_orientation (HildonDesktopPanelWindow *window, gint px, gint py);
+
+static gboolean hildon_desktop_panel_press_button_event (GtkWidget *widget, 
+                                                         GdkEventButton *event);
+
+static gboolean hildon_desktop_panel_release_button_event (GtkWidget *widget, 
+                                                           GdkEventButton *event);
+
+static gboolean hildon_desktop_panel_motion_notify_event (GtkWidget *widget, 
+                                                          GdkEventMotion *event);
+
+static void hildon_desktop_panel_size_request (GtkWidget *widget, 
+                                               GtkRequisition *requisition);
+
+static void hildon_desktop_panel_size_allocate (GtkWidget *widget, 
+                                                GtkAllocation *allocation);
+
+static void hildon_desktop_panel_stop_grab (HildonDesktopPanelWindow *window, 
+                                            guint32 timestamp);
+
+static gboolean hildon_desktop_panel_grab_motion_event (HildonDesktopPanelWindow *window, 
+                                                        GdkEventMotion *event);
+
+static void hildon_desktop_panel_calc_orientation (HildonDesktopPanelWindow *window, 
+                                                   gint px, 
+                                                   gint py);
+
 static void hildon_desktop_panel_begin_grab (HildonDesktopPanelWindow *window,
                                              HildonDesktopPanelWindowState state,
                                              gboolean grab_keyboard,
                                              guint32 timestamp);
+static void hildon_desktop_panel_win_move_resize (HildonDesktopPanelWindow *window,gboolean move,gboolean resize);
 
 GType
 hildon_desktop_panel_window_orientation_get_type (void)
@@ -166,6 +205,16 @@ hildon_desktop_panel_window_class_init (HildonDesktopPanelWindowClass *dskwindow
 
   container_class->check_resize = hildon_desktop_panel_window_check_resize;
   
+  signals[SIGNAL_ORIENTATION_CHANGED] =
+        g_signal_new ("orientation-changed",
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_FIRST,
+		      G_STRUCT_OFFSET (HildonDesktopPanelWindowClass, orientation_changed),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__INT,
+                      G_TYPE_NONE, 1,
+                      G_TYPE_INT);
+
   g_object_class_install_property (object_class,
 		  		   PROP_HOR_W,
 				   g_param_spec_int(
@@ -336,6 +385,12 @@ hildon_desktop_panel_window_init (HildonDesktopPanelWindow *window)
   /* FIXME: what do we do with focus??? */
 }
 
+static void 
+hildon_desktop_panel_window_force_move (GtkWidget *widget, GdkEventExpose *event, gpointer data)
+{
+  hildon_desktop_panel_win_move_resize (HILDON_DESKTOP_PANEL_WINDOW (widget),TRUE,TRUE);
+}
+
 static GObject *  
 hildon_desktop_panel_window_constructor (GType gtype,
 			                 guint n_params,
@@ -343,7 +398,8 @@ hildon_desktop_panel_window_constructor (GType gtype,
 {
   GObject *object;
   HildonDesktopPanelWindow *window;
-  GtkWidget *widget;
+  GtkWidget *widget, *alignment;
+  gint padding_left = 0, padding_right = 0, padding_top = 0, padding_bottom = 0;
   
   object = G_OBJECT_CLASS (hildon_desktop_panel_window_parent_class)->constructor (gtype,
 		  				                                   n_params,
@@ -352,6 +408,7 @@ hildon_desktop_panel_window_constructor (GType gtype,
   widget = GTK_WIDGET (object);
   window = HILDON_DESKTOP_PANEL_WINDOW (object);
 
+  
   GTK_WINDOW (window)->type = GTK_WINDOW_TOPLEVEL;
 
   gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_DOCK);
@@ -370,11 +427,36 @@ hildon_desktop_panel_window_constructor (GType gtype,
 		  "orientation",
 		  GTK_ORIENTATION_VERTICAL,
 		  NULL);
+
+  g_object_get (G_OBJECT (window),
+                "padding-left", &padding_left,
+                "padding-right", &padding_right,
+                "padding-top", &padding_top,
+                "padding-bottom", &padding_bottom,
+                NULL);
+
+  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
+
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 
+                             padding_top,
+                             padding_bottom,
+                             padding_left,
+                             padding_right);
   
-  gtk_container_add (GTK_CONTAINER (window),
+  gtk_widget_show (alignment);
+
+  gtk_container_add (GTK_CONTAINER (alignment),
 		     GTK_WIDGET (HILDON_DESKTOP_WINDOW (window)->container));
+
+  gtk_container_add (GTK_CONTAINER (window), alignment);
+
   gtk_widget_show (GTK_WIDGET (HILDON_DESKTOP_WINDOW (window)->container));
 
+  g_signal_connect_after (object,
+		          "expose-event",
+			  G_CALLBACK (hildon_desktop_panel_window_force_move),
+			  NULL);
+
   gtk_widget_pop_composite_child ();
 
   return object;
@@ -523,9 +605,12 @@ hildon_desktop_panel_win_set_orientation (HildonDesktopPanelWindow *window,
 
   if (orientation != window->priv->orientation)
     change_position = TRUE;
-
+    
   if (change_position)  
+  {
     window->priv->orientation = orientation;
+    g_signal_emit (window, signals[SIGNAL_ORIENTATION_CHANGED], 0, orientation);
+  }
 
   if (HILDON_DESKTOP_WINDOW (window)->container && rotate)
   {
