@@ -36,7 +36,8 @@ enum
   PROP_0,
   PROP_ORI,
   PROP_ITEM_WIDTH,
-  PROP_ITEM_HEIGHT
+  PROP_ITEM_HEIGHT,
+  PROP_PACK_START
 };
 
 enum
@@ -124,8 +125,8 @@ hildon_desktop_panel_class_init (HildonDesktopPanelClass *panel_class)
                      G_SIGNAL_RUN_FIRST,
 		     G_STRUCT_OFFSET (HildonDesktopPanelClass,add_button),
                      NULL, NULL,
-                     g_cclosure_marshal_VOID__POINTER,
-                     G_TYPE_NONE, 0);
+                     g_cclosure_marshal_VOID__OBJECT,
+                     G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
  
 
   g_object_class_install_property (object_class,
@@ -157,6 +158,15 @@ hildon_desktop_panel_class_init (HildonDesktopPanelClass *panel_class)
                                                      G_MAXINT,
                                                      0,
                                                      G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class,
+                                   PROP_PACK_START,
+                                   g_param_spec_boolean  ("pack_start",
+                                                          "packstart",
+                                                           "pack start or pack end",
+                                                           TRUE,
+                                                           G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
+
 }
 
 static void 
@@ -216,6 +226,10 @@ hildon_desktop_panel_get_property (GObject *object,
       g_value_set_int (value, panel->item_height);
       break;
 
+    case PROP_PACK_START:
+      g_value_set_boolean (value, panel->pack_start);
+      break;
+      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;						
@@ -248,12 +262,16 @@ hildon_desktop_panel_set_property (GObject *object,
       break;
       
     case PROP_ITEM_WIDTH:
-      panel->item_width = g_value_get_int (value);
+      panel->item_width  = g_value_get_int (value);
       break;
       
     case PROP_ITEM_HEIGHT:
       panel->item_height = g_value_get_int (value);
       break;
+
+    case PROP_PACK_START:
+      panel->pack_start  = g_value_get_boolean (value);
+      break;
       
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -762,7 +780,11 @@ hildon_desktop_panel_calc_positions (HildonDesktopPanel *panel,
 {
   /* FIXME: Please, implement me smoothly and very optimized */
   /* FIXME: This is not that smooth implementation, this only add the item*/
-  gtk_box_pack_start (GTK_BOX (panel), GTK_WIDGET (item), FALSE, FALSE,0);
+  if (panel->pack_start)
+    gtk_box_pack_start (GTK_BOX (panel), GTK_WIDGET (item), FALSE, FALSE,0);
+  else
+    gtk_box_pack_end   (GTK_BOX (panel), GTK_WIDGET (item), FALSE, FALSE, 0);
+
   gtk_widget_show    (GTK_WIDGET (item)); 
 }
 
@@ -821,7 +843,10 @@ hildon_desktop_panel_real_add_button (HildonDesktopPanel *panel, GtkWidget *widg
   }
   else
   {
-    gtk_box_pack_start (GTK_BOX (panel), widget, FALSE, FALSE, 0);
+    if (panel->pack_start)
+      gtk_box_pack_start (GTK_BOX (panel), widget, FALSE, FALSE, 0);
+    else
+      gtk_box_pack_end   (GTK_BOX (panel), widget, FALSE, FALSE, 0);
   }
 }
 /* Public declarations */
