@@ -22,6 +22,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "hildon-home-applet.h"
 #include "hildon-home-area.h"
@@ -29,9 +32,12 @@
 
 #include <gtk/gtkfixed.h>
 #include <gtk/gtkicontheme.h>
+
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-defines.h>
+#else
 #include <hildon-widgets/hildon-defines.h>
-
-
+#endif
 
 #define APPLET_RESIZE_HANDLE_ICON   "qgn_home_layoutmode_resize"
 #define APPLET_RESIZE_HANDLE_WIDTH  40
@@ -605,10 +611,6 @@ hildon_home_applet_unrealize (GtkWidget *widget)
       priv->event_window = NULL;
     }
 
-  else
-    g_object_unref (widget->window);
-
-
   if (GTK_WIDGET_CLASS (parent_class)->unrealize)
     GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 
