@@ -85,6 +85,9 @@ gboolean    hildon_home_area_get_layout_changed (HildonHomeArea *area);
 
 gboolean    hildon_home_area_get_overlaps       (HildonHomeArea *area);
 
+void        hildon_home_area_set_batch_add      (HildonHomeArea *area,
+                                                 gboolean batch_add);
+
 
 G_END_DECLS
 
