@@ -44,22 +44,68 @@
 
 #define MCE_TRIGGER_POWERKEY_EVENT_REQ  "req_trigger_powerkey_event"
 
+G_DEFINE_TYPE (HDKeysConfig, hd_keys_config, G_TYPE_OBJECT);
+
+static void hd_keys_config_finalize (GObject *object);
+
+static void gconf_key_changed_callback (GConfClient *client,
+					guint connection_id,
+					GConfEntry  *entry,gpointer
+				       	user_data);
+
+static void hd_keys_set_modifiers (HDKeysConfig *keys);
+
+static void hd_keys_load_and_grab_all_shortcuts (HDKeysConfig *keys);
+
+static void 
+hd_keys_config_class_init (HDKeysConfigClass *keys_class)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (keys_class);
+
+  object_class->finalize = hd_keys_config_finalize;
+}
+
+static void 
+hd_keys_config_init (HDKeysConfig *keys)
+{
+  keys->gconf_client  = gconf_client_get_default();
+
+  gconf_client_add_dir (keys->gconf_client,
+			HD_KEYS_GCONF_PATH,
+			GCONF_CLIENT_PRELOAD_NONE,
+			NULL);
+
+  gconf_client_notify_add (keys->gconf_client, 
+			   HD_KEYS_GCONF_PATH,
+			   gconf_key_changed_callback,
+			   keys,
+			   NULL, 
+			   NULL);
+  
+  hd_keys_set_modifiers (keys);
+
+  hd_keys_load_and_grab_all_shortcuts (keys);
+
+  g_debug ("**** Shortcuts loaded and grabbed ****");
+}
+
+static void 
+hd_keys_config_finalize (GObject *object)
+{
+  HDKeysConfig *keys = HD_KEYS_CONFIG (object);
+
+  g_slist_free (keys->shortcuts);
+
+  g_object_unref (keys->gconf_client);
+}
+
 static void 
 hd_keys_action_send_key (HDKeysConfig *keys,
 			 gpointer     *user_data)
 {
-  KeyCode keycode = 0;
   KeySym  keysym = (KeySym)user_data;
-  
-  HN_DBG("Faking keysym %li", keysym);
 
-  if ((keycode = XKeysymToKeycode (GDK_DISPLAY(), keysym)) != 0)
-    {
-        XTestFakeKeyEvent (GDK_DISPLAY(), keycode, TRUE, CurrentTime);
-        XTestFakeKeyEvent (GDK_DISPLAY(), keycode, FALSE, CurrentTime);
-
-	XSync(GDK_DISPLAY(), False);
-    }
+  hd_keys_send_key_by_keysym (keys,keysym);
 }
 
 static void 
@@ -69,13 +115,9 @@ hd_keys_action_close (HDKeysConfig *keys,
   HDWMWatchedWindow * win = hd_wm_get_active_window();
 
   if (win)
-    {
-      hd_wm_watched_window_close (win);
-    }
+    hd_wm_watched_window_close (win);
   else
-    {
-      g_warning ("No active window set");
-    }
+    g_warning ("No active window set");
 }
 
 static void 
@@ -86,25 +128,22 @@ hd_keys_action_minimize (HDKeysConfig *keys,
   HDWMWatchedWindow * win = hd_wm_get_active_window();
 
   if (win)
-    {
-      XWindowAttributes attrs;
-      Window xid = hd_wm_watched_window_get_x_win (win);
-      XGetWindowAttributes (GDK_DISPLAY(), xid, &attrs);
+  {
+    XWindowAttributes attrs;
+    Window xid = hd_wm_watched_window_get_x_win (win);
+    XGetWindowAttributes (GDK_DISPLAY(), xid, &attrs);
       
-      XIconifyWindow (GDK_DISPLAY(), xid,
-                      XScreenNumberOfScreen (attrs.screen));      
-    }
+    XIconifyWindow (GDK_DISPLAY(), xid, XScreenNumberOfScreen (attrs.screen));      
+  }
   else
-    {
-      g_warning ("No active window set");
-    }
+    g_warning ("No active window set");
 }
 
 static void 
 hd_keys_action_tn_activate (HDKeysConfig *keys,
                             gpointer     *user_data)
 {
-  hd_wm_activate(GPOINTER_TO_INT (user_data));
+  hd_wm_activate (GPOINTER_TO_INT (user_data));
 }
 
 static void 
@@ -120,26 +159,26 @@ hd_keys_action_power (HDKeysConfig *keys,
                       gpointer     *user_data)
 {
   osso_rpc_t       retval;
-  osso_context_t * osso_context
-    = get_context(osso_manager_singleton_get_instance());
+  
+  osso_context_t * osso_context =
+    get_context(osso_manager_singleton_get_instance());
+  
   gboolean long_press = GPOINTER_TO_INT(user_data);
   
   if (!osso_context ||
-      OSSO_OK != osso_rpc_run_system (osso_context,
-                                      MCE_SERVICE,
-                                      MCE_REQUEST_PATH,
-                                      MCE_REQUEST_IF,
-                                      MCE_TRIGGER_POWERKEY_EVENT_REQ,
-                                      &retval,
-                                      DBUS_TYPE_BOOLEAN, long_press,
-                                      DBUS_TYPE_INVALID))
-    {
-          g_warning("Unable to toggle Power menu");
-    }
+    OSSO_OK != osso_rpc_run_system (osso_context,
+                                    MCE_SERVICE,
+                                    MCE_REQUEST_PATH,
+                                    MCE_REQUEST_IF,
+                                    MCE_TRIGGER_POWERKEY_EVENT_REQ,
+                                    &retval,
+                                    DBUS_TYPE_BOOLEAN, long_press,
+                                    DBUS_TYPE_INVALID))
+  {
+    g_warning ("Unable to toggle Power menu");
+  }
 }
 
-/* ==================================================================== */
-
 struct 
 { 
   char            *gkey; 
@@ -232,45 +271,52 @@ hd_keys_set_modifiers (HDKeysConfig *keys)
   kpm = mod_map->max_keypermod;
 
   for (mod_idx = 0; mod_idx < 8; mod_idx++)
+  {	 
     for (mod_key = 0; mod_key < kpm; mod_key++) 
+    {
+      KeySym last_sym = 0;
+      for (col = 0; col < 4; col += 2) 
       {
-	KeySym last_sym = 0;
-	for (col = 0; col < 4; col += 2) 
-	  {
-	    KeyCode code = mod_map->modifiermap[mod_idx * kpm + mod_key];
-	    KeySym sym = (code ? XKeycodeToKeysym(GDK_DISPLAY(), code, col) : 0);
+        KeyCode code = mod_map->modifiermap[mod_idx * kpm + mod_key];
+        KeySym sym = (code ? XKeycodeToKeysym (GDK_DISPLAY(), code, col) : 0);
 
-	    if (sym == last_sym) continue;
-	    last_sym = sym;
+        if (sym == last_sym) continue;
+	
+        last_sym = sym;
 
-	    switch (sym) 
-	      {
-	      case XK_Meta_L:
-	      case XK_Meta_R:
-		keys->meta_mask |= (1 << mod_idx); 
-		break;
-	      case XK_Super_L:
-	      case XK_Super_R:
-		keys->super_mask |= (1 << mod_idx);
-		break;
-	      case XK_Hyper_L:
-	      case XK_Hyper_R:
-		keys->hyper_mask |= (1 << mod_idx);
-		break;
-	      case XK_Alt_L:
-	      case XK_Alt_R:
-		keys->alt_mask |= (1 << mod_idx);
-		break;
-	      case XK_Num_Lock:
-		keys->numlock_mask |= (1 << mod_idx);
-		break;
-	      case XK_Scroll_Lock:
-		keys->scrolllock_mask |= (1 << mod_idx);
-		break;
-	      }
-	  }
+         switch (sym) 
+         {
+           case XK_Meta_L:
+           case XK_Meta_R:
+     	     keys->meta_mask |= (1 << mod_idx); 
+	     break;
+	     
+	   case XK_Super_L:
+	   case XK_Super_R:
+	     keys->super_mask |= (1 << mod_idx);
+	     break;
+	     
+	   case XK_Hyper_L:
+	   case XK_Hyper_R:
+	     keys->hyper_mask |= (1 << mod_idx);
+	     break;
+	     
+	   case XK_Alt_L:
+	   case XK_Alt_R:
+	     keys->alt_mask |= (1 << mod_idx);
+	     break;
+	     
+	   case XK_Num_Lock:
+	     keys->numlock_mask |= (1 << mod_idx);
+	     break;
+	   
+	   case XK_Scroll_Lock:
+	     keys->scrolllock_mask |= (1 << mod_idx);
+	     break;
+	}
       }
-
+    }
+  }
   /* Assume alt <=> meta if only either set */
   if (!keys->alt_mask)  keys->alt_mask  = keys->meta_mask; 
   if (!keys->meta_mask) keys->meta_mask = keys->alt_mask; 
@@ -310,204 +356,219 @@ hd_keys_shortcut_new (HDKeysConfig *keys,
   };
 
   while (*p != '\0')
+  {
+    Bool found = False;
+
+   /* Parse <modifier> and look up its mask
+    */
+    if (*p == '<' && *(p+1) != '\0') 
     {
-      Bool found = False;
-
-      /* Parse <modifier> and look up its mask
-      */
-      if (*p == '<' && *(p+1) != '\0') 
-	{
-	  q = ++p;
-	  while (*q != '\0' && *q != '>') q++;
-	  if (*q == '\0') 
-	    return NULL;
+      q = ++p;
+      while (*q != '\0' && *q != '>') q++;
+      if (*q == '\0') 
+        return NULL;
 	  
-	  i = 0;
+      i = 0;
 
-	  while (mod_lookup[i].def != NULL && !found)
-	    {
-	      if (!g_ascii_strncasecmp (p, mod_lookup[i].def, q-p))
-		{
-		  if (mod_lookup[i].mask)
-		    {
-		      mask |= mod_lookup[i].mask;
-		      found = True;
-		    } 
-		}
-	      i++;
-	    }
-
-	  if (found) 
-	    {
-	      p = q; /* found the modifier skip index over tag  */
-	    } 
-	  else 
-	    {
-	      /* Check if its <shift> */
-	      if (!g_ascii_strncasecmp (p, "shift", 5))
-		{
-		  want_shift = True;
-		  p = q;
-		}
-	      else return NULL; /* Unknown modifier string */
-	    }
-
-	} 
-      /* Now parse the actual Key */
-      else if (!g_ascii_isspace(*p)) 
-	{
-	  keydef = p;
-	  break;
+      while (mod_lookup[i].def != NULL && !found)
+      {
+        if (!g_ascii_strncasecmp (p, mod_lookup[i].def, q-p))
+ 	{
+ 	  if (mod_lookup[i].mask)
+          {
+            mask |= mod_lookup[i].mask;
+            found = True;
+    	  } 
 	}
-      p++;
+	
+  	i++;
+      }
+
+      if (found) 
+      {
+        p = q; /* found the modifier skip index over tag  */
+      } 
+      else 
+      {
+        /* Check if its <shift> */
+        if (!g_ascii_strncasecmp (p, "shift", 5))
+	{
+	  want_shift = True;
+	  p = q;
+	}
+        else 
+	  return NULL; /* Unknown modifier string */
+      }
+
+    } 
+    /* Now parse the actual Key */
+    else 
+    if (!g_ascii_isspace(*p)) 
+    {
+      keydef = p;
+      break;
     }
+    
+    p++;  
+  }
 
   if (!keydef) 
     return NULL;
 
   if ((ks = XStringToKeysym(keydef)) == (KeySym)NULL)
-    {
-      if ( g_ascii_islower(keydef[0]))  /* Try again, changing case */
-	keydef[0] = g_ascii_toupper(keydef[0]);
-      else
-	keydef[0] = g_ascii_tolower(keydef[0]);
+  {
+    if (g_ascii_islower(keydef[0]))  /* Try again, changing case */
+      keydef[0] = g_ascii_toupper(keydef[0]);
+    else
+      keydef[0] = g_ascii_tolower(keydef[0]);
 
-      if ((ks = XStringToKeysym(keydef)) == (KeySym)NULL)
-	  return NULL; /* Couldn't find a keysym */
-    }
+    if ((ks = XStringToKeysym(keydef)) == (KeySym)NULL)
+      return NULL; /* Couldn't find a keysym */
+  }
 
   /* Cannot install grab for keys that we are supposed to fake */
   switch (ks)
-    {
-      case XK_F4:
-      case XK_F6:
-      case XK_F7:
-      case XK_F8:
-        HN_DBG ("Illegal shortcut symbol -- ignoring");
-        return NULL;
-      
-      default:
-        ;
+  {
+    case XK_F4:
+    case XK_F6:
+    case XK_F7:
+    case XK_F8:
+      g_debug ("Illegal shortcut symbol -- ignoring");
+      return NULL;
+    default:
+      g_debug ("%s: %d, should never be reached",__FILE__,__LINE__);
     }
   
   if (hd_keys_keysym_needs_shift(ks))
+  {
+    mask |= ShiftMask;
+    index = 1;
+  }
+  else 
+  if (want_shift)
+  {
+    KeySym shifted;
+      
+    /* Change the keysym for case of '<shift>lowerchar' where we  
+     * actually want to grab the shifted version of the keysym.
+     */
+    if ((shifted = hd_keys_keysym_get_shifted (ks)) != NoSymbol)
     {
-      mask |= ShiftMask;
+      ks = shifted;
       index = 1;
     }
-  else if (want_shift)
-    {
-      KeySym shifted;
-      
-      /* Change the keysym for case of '<shift>lowerchar' where we  
-       * actually want to grab the shifted version of the keysym.
-      */
-      if ((shifted = hd_keys_keysym_get_shifted (ks)) != NoSymbol)
-        {
-          ks = shifted;
-          index = 1;
-        }
 
-      /* Set the mask even if no shifted version - <shift>up key for  
-       * example. 
-      */
-      mask |= ShiftMask;
-    }
+    /* Set the mask even if no shifted version - <shift>up key for  
+     * example. 
+    */
+    mask |= ShiftMask;
+  }
 
   /* If F5 is assigned to "Home", don't do anything or we will be
    * conflicting with MCE's handling of the key */
-  if (ks == XK_F5 && 
-          HDKeysActionConfLookup[conf_index].action == HD_KEY_ACTION_HOME)
-      return NULL;
+  if (ks == XK_F5 && HDKeysActionConfLookup[conf_index].action == HD_KEY_ACTION_HOME)
+    return NULL;
   
   /* If we grab keycode 0, we end up grabbing the entire keyboard :\ */
   if (XKeysymToKeycode(GDK_DISPLAY(), ks) == 0)
-      return NULL;
+    return NULL;
 
   shortcut = g_new0(HDKeyShortcut, 1);
-  shortcut->action          
-    = HDKeysActionConfLookup[conf_index].action;
-  shortcut->action_func     
-    = HDKeysActionConfLookup[conf_index].action_func;
-  shortcut->action_func_data 
-    = HDKeysActionConfLookup[conf_index].action_func_data;
+  
+  shortcut->action           = HDKeysActionConfLookup[conf_index].action;
+  shortcut->action_func      = HDKeysActionConfLookup[conf_index].action_func;
+  shortcut->action_func_data = HDKeysActionConfLookup[conf_index].action_func_data;
+  
   shortcut->mod_mask = mask;
   shortcut->keysym   = ks;
-  shortcut->keycode  = XKeysymToKeycode(GDK_DISPLAY(), ks);
+  gdk_keymap_get_entries_for_keyval (NULL /* default keymap */,
+				     ks,
+				     &shortcut->keycodes,
+				     &shortcut->n_keycodes);
   shortcut->index    = index;
 
-  HN_DBG("'%s' to new shortcut with ks:%li, mask:%i",
-	 keystr, ks, mask);
+  g_debug("'%s' to new shortcut with ks:%li, mask:%i", keystr, ks, mask);
 
   return shortcut;
 }
 
+static void
+hd_keys_shortcut_free (HDKeyShortcut *shortcut)
+{
+  g_free (shortcut->keycodes);
+  g_free (shortcut);
+}
 
 static gboolean
-hn_key_shortcut_grab (HDKeysConfig  *keys, 
+hd_key_shortcut_grab (HDKeysConfig  *keys, 
 		      HDKeyShortcut *shortcut,
 		      gboolean       ungrab)
 {
-  int ignored_mask = 0;
+  gint ignored_mask = 0;
+  gint i_keycode;
+
+  if (!shortcut->keycodes)
+    return FALSE;
 
   /* Needed to grab all ignored combo's too if num of scroll lock are on */
-  while (ignored_mask < (int) keys->lock_mask)
+  for (i_keycode = 0; i_keycode < shortcut->n_keycodes; i_keycode++)
+  {
+    while (ignored_mask < (int) keys->lock_mask)
     {                                       
       if (ignored_mask & ~(keys->lock_mask))
-	{
-	  ++ignored_mask;
-	  continue;
-	}
+      {
+        ++ignored_mask;
+        continue;
+      }
 
+      
       if (ungrab)
-	{
-	  XUngrabKey(GDK_DISPLAY(),
-		     shortcut->keycode, 
-		     shortcut->mod_mask | ignored_mask,
-		     GDK_ROOT_WINDOW());
-	} 
-      else 
-	{
-	  int result; 
-
-	  gdk_error_trap_push();	  
-	  
-	  XGrabKey (GDK_DISPLAY(), 
-                    shortcut->keycode, 
+      {
+        XUngrabKey (GDK_DISPLAY(),
+	            shortcut->keycodes[i_keycode].keycode, 
 		    shortcut->mod_mask | ignored_mask,
-		    GDK_ROOT_WINDOW(), 
-		    False, 
-		    GrabModeAsync, 
-		    GrabModeAsync);
-	  
-	  XSync(GDK_DISPLAY(), False);
-	  result = gdk_error_trap_pop();
+		    GDK_ROOT_WINDOW());
+      } 
+      else 
+      {
+        gint result; 
 
-	  if (result)
-	    {
-	      /* FIXME: Log below somewhere */
-	      if (result == BadAccess)
-		{
-		  fprintf(stderr, 
-			  "Some other program is already using the key %s "
-			  "with modifiers %x as a binding\n",  
-			  (XKeysymToString (shortcut->keysym)) ? 
-			    XKeysymToString (shortcut->keysym) : "unknown", 
-			    shortcut->mod_mask | ignored_mask );
-		}
-	      else
-		{
-		  fprintf(stderr, 
-			  "Unable to grab the key %s with modifiers %x"
-			  "as a binding\n",  
-			  (XKeysymToString (shortcut->keysym)) ? 
-			    XKeysymToString (shortcut->keysym) : "unknown", 
-			    shortcut->mod_mask | ignored_mask );
-		}
-	    }
-	}
+        gdk_error_trap_push();	  
+	  
+        XGrabKey (GDK_DISPLAY(),
+		  shortcut->keycodes[i_keycode].keycode,
+		  shortcut->mod_mask | ignored_mask,
+		  GDK_ROOT_WINDOW(), 
+		  False, 
+		  GrabModeAsync, 
+		  GrabModeAsync);
+	  
+        XSync(GDK_DISPLAY(), False);
+
+        result = gdk_error_trap_pop();
+
+        if (result)
+        {
+          /* FIXME: Log below somewhere */
+          if (result == BadAccess)
+	  {
+	    g_debug ("Some other program is already using the key %s "
+	  	     "with modifiers %x as a binding\n",  
+ 		     (XKeysymToString (shortcut->keysym)) ? XKeysymToString (shortcut->keysym) : "unknown", 
+	             shortcut->mod_mask | ignored_mask );
+	  }
+          else
+	  {
+	    g_debug ("Unable to grab the key %s with modifiers %x as a binding\n", 
+		     (XKeysymToString (shortcut->keysym)) ? XKeysymToString (shortcut->keysym) : "unknown", 
+		      shortcut->mod_mask | ignored_mask);
+          }
+        }
+      }  
       ++ignored_mask;
     }
+    ignored_mask = 0;
+  }
   
   return TRUE;
 }
@@ -519,25 +580,25 @@ hd_keys_load_and_grab_all_shortcuts (HDKeysConfig *keys)
   char *key_def_str = NULL;
 
   while (HDKeysActionConfLookup[i].gkey != NULL)
-    {
-      key_def_str = gconf_client_get_string(keys->gconf_client, 
-					    HDKeysActionConfLookup[i].gkey, 
-					    NULL);
+  {
+    key_def_str = 
+      gconf_client_get_string (keys->gconf_client, 
+		               HDKeysActionConfLookup[i].gkey,
+			       NULL);
       
-      if (key_def_str && g_ascii_strncasecmp(key_def_str, "disabled", 8))
-	{
-	  HDKeyShortcut *shortcut;
+    if (key_def_str && g_ascii_strncasecmp(key_def_str, "disabled", 8))
+    {
+      HDKeyShortcut *shortcut;
 	  
-	  if ((shortcut = hd_keys_shortcut_new (keys, key_def_str, i)) != NULL)
-	    {
-	      HN_DBG("Grabbing '%s'", key_def_str);
-	      hn_key_shortcut_grab (keys, shortcut, FALSE); 
-	      keys->shortcuts = g_slist_append (keys->shortcuts, shortcut);
-	    }
-	}
-
-      i++;
+      if ((shortcut = hd_keys_shortcut_new (keys, key_def_str, i)) != NULL)
+      {
+        g_debug ("Grabbing '%s'", key_def_str);
+        hd_key_shortcut_grab (keys, shortcut, FALSE); 
+        keys->shortcuts = g_slist_append (keys->shortcuts, shortcut);
+      }
     }
+    i++;
+  }
 }
 
 static void
@@ -551,98 +612,80 @@ gconf_key_changed_callback (GConfClient *client,
 
   gvalue = gconf_entry_get_value(entry);
   
-  HN_DBG("called");
+  g_debug ("called");
 
   if (gconf_entry_get_key(entry))
-    {
-      /* Parse, find entry, ungrab, replace, regrab */
-      HDKeyShortcut *shortcut;
-      gint           i = 0;
-      GSList        *item;
-      const gchar   *value;
+  {
+    /* Parse, find entry, ungrab, replace, regrab */
+    HDKeyShortcut *shortcut;
+    gint           i = 0;
+    GSList        *item;
+    const gchar   *value;
 
-      HN_DBG("Looking up '%s'", gconf_entry_get_key(entry));
+    g_debug ("Looking up '%s'", gconf_entry_get_key(entry));
 
       /* Find what this keys matches in lookup */
-      while (HDKeysActionConfLookup[i].gkey != NULL)
-	{
-	  if (g_str_equal(gconf_entry_get_key(entry),  
-			  HDKeysActionConfLookup[i].gkey))
-	    break;
-	  i++;
-	}
+    while (HDKeysActionConfLookup[i].gkey != NULL)
+    {
+      if (g_str_equal(gconf_entry_get_key(entry), HDKeysActionConfLookup[i].gkey))
+        break;
+      i++;
+    }
 
-      if (HDKeysActionConfLookup[i].gkey == NULL)
-	{
-	  HN_DBG("Unable to find '%s'", gconf_entry_get_key(entry));
-	  return; 		/* key not found */
-	}
+    if (HDKeysActionConfLookup[i].gkey == NULL)
+    {
+      g_debug ("Unable to find '%s'", gconf_entry_get_key(entry));
+      return; 		/* key not found */
+    }
 
-      item = keys->shortcuts;
+    item = keys->shortcuts;
 
       /* Remove a potential existing grab and shortcut for this action */
-      while (item)
-	{
-	  HDKeyShortcut *sc = (HDKeyShortcut *)item->data;
+    while (item)
+    {
+      HDKeyShortcut *sc = (HDKeyShortcut *)item->data;
 
-	  if (sc->action == HDKeysActionConfLookup[i].action)
-	    {
-	      HN_DBG("removing exisiting action %i", sc->action);
-	      hn_key_shortcut_grab (keys, sc, TRUE); 
-	      keys->shortcuts = g_slist_remove (keys->shortcuts, item->data);
-	      g_free (sc);
-	      break;
-	    }
-	  item = g_slist_next(item);
-	}
+      if (sc->action == HDKeysActionConfLookup[i].action)
+      {
+        g_debug ("removing exisiting action %i", sc->action);
+        hd_key_shortcut_grab (keys, sc, TRUE); 
+        keys->shortcuts = g_slist_remove (keys->shortcuts, item->data);
+        hd_keys_shortcut_free (sc);
+        break;
+      }
+      item = g_slist_next(item);
+    }
 
       /* If value is not NULL to handle removing of key  */
-      if (gvalue && gvalue->type == GCONF_VALUE_STRING)
-	{
-	  value = gconf_value_get_string (gvalue);
+    if (gvalue && gvalue->type == GCONF_VALUE_STRING)
+    {
+      value = gconf_value_get_string (gvalue);
 
-	  HN_DBG("attempting to grab '%s'", value);
+      g_debug ("attempting to grab '%s'", value);
 
-	  /* Grab the new shortcut and addd to our list */
-	  if (value && g_ascii_strncasecmp(value, "disabled", 8))
-	    if ((shortcut = hd_keys_shortcut_new (keys, value, i)) != NULL)
-	      {
-		HN_DBG("now grabbing '%s'", value);
-		hn_key_shortcut_grab (keys, shortcut, FALSE); 
-		keys->shortcuts = g_slist_append (keys->shortcuts, shortcut);
-	      }
-	} 
-    }
+      /* Grab the new shortcut and addd to our list */
+      if (value && g_ascii_strncasecmp(value, "disabled", 8))
+      {
+        if ((shortcut = hd_keys_shortcut_new (keys, value, i)) != NULL)
+        {
+  	  g_debug ("now grabbing '%s'", value);
+	  hd_key_shortcut_grab (keys, shortcut, FALSE); 
+	  keys->shortcuts = g_slist_append (keys->shortcuts, shortcut);
+	}
+      }
+    } 
+  }
 }
 
 HDKeysConfig*
-hd_keys_init (void)
+hd_keys_config_get_singleton (void)
 {
-  HDKeysConfig *keys;
+  static HDKeysConfig *singleton = NULL;
 
-  keys = g_new0 (HDKeysConfig, 1);
+  if (!singleton)
+    singleton = HD_KEYS_CONFIG (g_object_new (HD_TYPE_KEYS_CONFIG,NULL));
 
-  keys->gconf_client  = gconf_client_get_default();
-
-  gconf_client_add_dir (keys->gconf_client,
-			HD_KEYS_GCONF_PATH,
-			GCONF_CLIENT_PRELOAD_NONE,
-			NULL);
-
-  gconf_client_notify_add (keys->gconf_client, 
-			   HD_KEYS_GCONF_PATH,
-			   gconf_key_changed_callback,
-			   keys,
-			   NULL, 
-			   NULL);
-  
-  hd_keys_set_modifiers (keys);
-
-  hd_keys_load_and_grab_all_shortcuts (keys);
-
-  HN_DBG("**** Shortcuts loaded and grabbed ****");
-
-  return keys;
+  return singleton;
 }
 
 void
@@ -653,12 +696,12 @@ hd_keys_reload (GdkKeymap *keymap, HDKeysConfig *keys)
   shortcut = keys->shortcuts;
   
   while (shortcut != NULL)
-    {
-      gpointer data = shortcut->data;
-      hn_key_shortcut_grab (keys, shortcut->data, TRUE);
-      shortcut = g_slist_remove (shortcut, shortcut->data);
-      g_free (data);
-    }
+  {
+    gpointer data = shortcut->data;
+    hd_key_shortcut_grab (keys, shortcut->data, TRUE);
+    shortcut = g_slist_remove (shortcut, shortcut->data);
+    hd_keys_shortcut_free (data);
+  }
 
   keys->shortcuts = NULL;
 
@@ -676,23 +719,43 @@ hd_keys_handle_keypress (HDKeysConfig *keys,
   item = keys->shortcuts;
 
   while (item)
-    {
-      HDKeyShortcut *shortcut = (HDKeyShortcut *)item->data;
+  {
+    HDKeyShortcut *shortcut = (HDKeyShortcut *)item->data;
 
-      HN_DBG("%i vs %i, %li vs %li",
+    g_debug ("%i vs %i, %li vs %li",
 	     shortcut->mod_mask, mod_mask,
 	     shortcut->keysym, XKeycodeToKeysym(GDK_DISPLAY(), keycode, 0));
 
-      if (shortcut->mod_mask == mod_mask &&
-	  shortcut->keysym == XKeycodeToKeysym(GDK_DISPLAY(), 
-                                               keycode, 
-                                               shortcut->index))
-	{
-	  return shortcut;
-	}
-      item = g_slist_next(item);
+    if (shortcut->mod_mask == mod_mask &&  
+	shortcut->keysym == XKeycodeToKeysym(GDK_DISPLAY(),keycode,shortcut->index))
+    {
+      return shortcut;
     }
+    
+    item = g_slist_next(item);
+  }
 
   return NULL;
 } 
 
+void 
+hd_keys_send_key_by_keysym (HDKeysConfig *keys, KeySym keysym)
+{
+  g_debug ("Faking keysym %li", keysym);
+
+  hd_keys_send_key_by_keycode (keys,XKeysymToKeycode (GDK_DISPLAY(), keysym));
+}
+
+void 
+hd_keys_send_key_by_keycode (HDKeysConfig *keys, KeyCode keycode)
+{
+  g_debug ("Faking keycode %d", keycode);
+
+  if (keycode != 0)
+  {
+    XTestFakeKeyEvent (GDK_DISPLAY(), keycode, TRUE, CurrentTime);
+    XTestFakeKeyEvent (GDK_DISPLAY(), keycode, FALSE, CurrentTime);
+
+    XSync(GDK_DISPLAY(), False);
+  }
+}
