@@ -1,8 +1,8 @@
 /* -*- mode:C; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
 /* 
- * This file is part of maemo-af-desktop
+ * This file is part of hildon-desktop
  *
- * Copyright (C) 2005, 2006 Nokia Corporation.
+ * Copyright (C) 2005, 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
@@ -22,6 +22,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/types.h>
 #include <signal.h>
 #include <string.h>
@@ -30,9 +34,17 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h> /* needed by hildon-navigator-main.h */
 #include <log-functions.h>
+
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-defines.h>
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-note.h>
+#else
 #include <hildon-widgets/hildon-defines.h>
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
+#endif
+
 #include "hd-wm-watched-window.h"
 #include "hd-wm-watched-window-view.h"
 #include "hd-wm-watchable-app.h"
