@@ -1,8 +1,8 @@
 /* -*- mode:C; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
 /* 
- * This file is part of maemo-af-desktop
+ * This file is part of hildon-desktop
  *
- * Copyright (C) 2006 Nokia Corporation.
+ * Copyright (C) 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
@@ -22,6 +22,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <string.h>
 #include <sys/types.h>
@@ -33,10 +36,18 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkevents.h>
 
+
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-defines.h>
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-note.h>
+#include <hildon/hildon-window.h>
+#else
 #include <hildon-widgets/hildon-defines.h>
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
 #include <hildon-widgets/hildon-window.h>
+#endif
 
 #include <hildon-base-lib/hildon-base-dnotify.h>
 
@@ -102,6 +113,7 @@ enum
   HDWM_ENTRY_INFO_STACK_CHANGED_SIGNAL,
   HDWM_WORK_AREA_CHANGED_SIGNAL,
   HDWM_SHOW_A_MENU_SIGNAL,
+  HDWM_APPLICATION_STARTING_SIGNAL,
   HDWM_SIGNALS
 };
 
@@ -401,12 +413,9 @@ hd_wm_dbus_method_call_handler (DBusConnection *connection,
 	  hdwm->priv->lowmem_banner_timeout > 0 &&
 	  !hd_wm_watchable_app_has_windows (app))
       {
-        g_debug ("Showing Launchbanner...");
-        /*
-	 * This function takes care of the distinction between the Loading
-	 * and Resuming banners, hence we no longer test for hibernation
-	 */
-	hd_wm_watchable_app_launch_banner_show (app);
+        g_signal_emit_by_name (hdwm,
+                               "application-starting",
+                               hd_wm_watchable_app_get_localized_name (app));
       }
     }
   }
@@ -694,6 +703,17 @@ hd_wm_class_init (HDWMClass *hdwm_class)
 		     NULL, NULL,
 		     g_cclosure_marshal_VOID__VOID,
 		     G_TYPE_NONE, 0);
+  
+  hdwm_signals[HDWM_APPLICATION_STARTING_SIGNAL] = 
+	g_signal_new("application_starting",
+		     G_OBJECT_CLASS_TYPE(object_class),
+		     G_SIGNAL_RUN_LAST,
+		     G_STRUCT_OFFSET (HDWMClass,application_starting),
+		     NULL, NULL,
+		     g_cclosure_marshal_VOID__STRING,
+		     G_TYPE_NONE,
+                     1,
+                     G_TYPE_STRING);
 
 }
 
