@@ -36,8 +36,16 @@
 
 #include <libhildondesktop/hildon-home-area.h>
 #include <libhildondesktop/hildon-home-titlebar.h>
+
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-note.h>
+#include <hildon/hildon-defines.h>
+#else
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
+#include <hildon-widgets/hildon-defines.h>
+#endif
 
 #include "hd-home-l10n.h"
 #include "hd-home-background.h"
@@ -124,9 +132,11 @@ hd_home_window_personalisation_activate (HDHomeWindow *window);
 static void
 hd_home_window_calibration_activate (HDHomeWindow *window);
 
+#ifdef HAVE_LIBOSSOHELP
 static void
 hd_home_window_help_activate (HDHomeWindow *window);
 #endif
+#endif
 
 static void
 hd_home_window_set_background_activate (HDHomeWindow *window);
@@ -166,10 +176,22 @@ static gboolean
 hd_home_window_map_event (GtkWidget    *widget,
                           GdkEventAny  *event);
 
+static gboolean
+hd_home_window_key_press_event (GtkWidget *widget,
+                                GdkEventKey *event);
+
 static void
 hd_home_window_style_set (GtkWidget    *widget,
                           GtkStyle     *style);
 
+static void
+hd_home_window_show_information_note (HDHomeWindow *window,
+                                      const gchar *text);
+
+static void
+hd_home_window_show_information_banner (HDHomeWindow *window,
+                                        const gchar *text);
+
 static void
 hd_home_window_class_init (HDHomeWindowClass *window_class)
 {
@@ -189,6 +211,7 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
 
   widget_class->map_event = hd_home_window_map_event;
   widget_class->style_set = hd_home_window_style_set;
+  widget_class->key_press_event = hd_home_window_key_press_event;
 
   hhwindow_class->layout_mode_accept = hd_home_window_layout_mode_accept;
   hhwindow_class->layout_mode_cancel = hd_home_window_layout_mode_cancel;
@@ -571,6 +594,38 @@ hd_home_window_style_set (GtkWidget *widget, GtkStyle *old_style)
     }
 }
 
+static gboolean
+hd_home_window_key_press_event (GtkWidget *widget,
+                                GdkEventKey *event)
+{
+  GtkWidget *titlebar = 
+      hildon_home_window_get_titlebar (HILDON_HOME_WINDOW (widget));
+  GtkWidget *area = 
+      hildon_home_window_get_area (HILDON_HOME_WINDOW (widget));
+
+  switch (event->keyval)
+    {
+      case HILDON_HARDKEY_MENU:
+          hildon_home_titlebar_toggle_menu (HILDON_HOME_TITLEBAR (titlebar));
+          break;
+      case HILDON_HARDKEY_ESC:
+          /* FIXME: Have a signal in HomeWindow instead */
+          if (hildon_home_area_get_layout_mode (HILDON_HOME_AREA(area)))
+            g_signal_emit_by_name (G_OBJECT (titlebar), "layout-cancel");
+
+          break;
+      default:
+          if (GTK_WIDGET_CLASS (hd_home_window_parent_class)->key_press_event)
+            return GTK_WIDGET_CLASS (
+                     hd_home_window_parent_class)->key_press_event (widget,
+                                                                    event);
+          else
+            return FALSE;
+    }
+
+  return TRUE;
+}
+
 static GtkWidget *
 hd_home_window_build_main_menu (HDHomeWindow *window)
 {
@@ -651,6 +706,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
   gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
   gtk_widget_show (menu_item);
 
+#ifdef HAVE_LIBOSSOHELP
   /* help */
   menu_item = gtk_menu_item_new_with_label (HH_MENU_HELP);
   gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
@@ -658,6 +714,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
                             G_CALLBACK (hd_home_window_help_activate),
                             window);
   gtk_widget_show (menu_item);
+#endif
 #endif
   
   return menu;
@@ -693,7 +750,7 @@ hd_home_window_build_layout_menu (HDHomeWindow *window)
                             window);
   gtk_widget_show (mi);
 
-#ifdef HAVE_LIBOSSO
+#ifdef HAVE_LIBOSSOHELP
   mi = gtk_menu_item_new_with_label (HH_MENU_LAYOUT_HELP);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate",
@@ -746,7 +803,7 @@ hd_home_window_applet_activate (HDHomeWindow *window,
 }
 #endif
 
-#ifdef HAVE_LIBOSSO
+#ifdef HAVE_LIBOSSOHELP
 static void
 hd_home_window_help_activate (HDHomeWindow *window)
 {
@@ -818,16 +875,16 @@ hd_home_window_personalisation_activate (HDHomeWindow *window)
 static void
 hd_home_window_settings_insensitive_press_cb (HDHomeWindow *window)
 {
-  hildon_home_window_show_information_banner (HILDON_HOME_WINDOW (window),
-                                              HH_APPLET_SETTINGS_BANNER);
+  hd_home_window_show_information_banner (window,
+                                          HH_APPLET_SETTINGS_BANNER);
 
 }
 
 static void
 hd_home_window_layout_insensitive_press_cb (HDHomeWindow *window)
 {
-  hildon_home_window_show_information_banner (HILDON_HOME_WINDOW (window),
-                                              HH_LAYOUT_UNAVAIL_BANNER);
+  hd_home_window_show_information_banner (window,
+                                          HH_LAYOUT_UNAVAIL_BANNER);
 
 }
 
@@ -912,9 +969,11 @@ hd_home_window_set_osso_context (HDHomeWindow *window,
 
     if (HILDON_IS_HOME_TITLEBAR (titlebar))
     {
+#ifdef HAVE_LIBOSSOHELP
       g_signal_connect_swapped (titlebar, "help-activate",
                                 G_CALLBACK (hd_home_window_help_activate),
                                 window);
+#endif
       
       g_signal_connect_swapped (titlebar, "applet-activate",
                                 G_CALLBACK (hd_home_window_applet_activate),
@@ -975,32 +1034,28 @@ hd_home_window_show_layout_mode_banner (HDHomeWindow *window)
 static void
 hd_home_window_layout_mode_accept (HildonHomeWindow *window)
 {
-  HildonHomeWindow *hhwindow;
   GtkWidget        *area;
 
-  hhwindow = HILDON_HOME_WINDOW (window);
-  area = hildon_home_window_get_area (hhwindow);
+  area = hildon_home_window_get_area (window);
 
   if (hildon_home_area_get_overlaps (HILDON_HOME_AREA (area)))
     {
-      hildon_home_window_show_information_note (window,
-                                                HH_LAYOUT_OVERLAP_TEXT);
+      hd_home_window_show_information_note (HD_HOME_WINDOW (window),
+                                            HH_LAYOUT_OVERLAP_TEXT);
       return;
     }
 
 
   if (HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_accept)
-    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_accept (hhwindow);
+    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_accept (window);
 }
 
 static void
 hd_home_window_layout_mode_cancel (HildonHomeWindow *window)
 {
-  HildonHomeWindow *hhwindow;
   GtkWidget        *area;
 
-  hhwindow = HILDON_HOME_WINDOW (window);
-  area = hildon_home_window_get_area (hhwindow);
+  area = hildon_home_window_get_area (window);
 
   if (hildon_home_area_get_layout_changed (HILDON_HOME_AREA (area)))
     {
@@ -1024,7 +1079,7 @@ hd_home_window_layout_mode_cancel (HildonHomeWindow *window)
     }
 
   if (HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_cancel)
-    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_cancel (hhwindow);
+    HILDON_HOME_WINDOW_CLASS (hd_home_window_parent_class)->layout_mode_cancel (window);
 }
 
 static void
@@ -1117,3 +1172,27 @@ hd_home_window_set_background_activate (HDHomeWindow *window)
   gtk_widget_show (dialog);
 
 }
+
+static void
+hd_home_window_show_information_note (HDHomeWindow *window,
+                                      const gchar *text)
+{
+  GtkWidget *note = NULL;
+
+  note = hildon_note_new_information (NULL, text);
+		    
+  gtk_dialog_run (GTK_DIALOG (note));
+  if (note) 
+    gtk_widget_destroy (GTK_WIDGET (note));
+}
+
+static void
+hd_home_window_show_information_banner (HDHomeWindow *window,
+                                        const gchar *text)
+{
+  g_return_if_fail (GTK_IS_WIDGET (window) && text);
+
+  hildon_banner_show_information (GTK_WIDGET (window),
+                                  NULL,
+                                  text);
+}
