@@ -65,11 +65,11 @@
 #include <libosso.h>
 #endif
 
-#ifndef HILDON_LIBS
+#ifdef HAVE_LIBHILDON
+#else
 #include <hildon-widgets/hildon-finger.h>
 #endif 
 
-#include <hildon-widgets/gtk-infoprint.h>
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
